@@ -12,47 +12,83 @@
 # implied. See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import copy
 import logging
+import typing as ty
 
-import debtcollector
 from oslo_config import cfg
 from oslo_middleware import base
 import webob.exc
 
+if ty.TYPE_CHECKING:
+    from _typeshed.wsgi import WSGIApplication
+    import webob.request
+    import webob.response
+
+
+class AllowedOrigin(ty.TypedDict):
+    allow_credentials: bool
+    expose_headers: list[str]
+    max_age: int | None
+    allow_methods: list[str]
+    allow_headers: list[str]
+
 
 LOG = logging.getLogger(__name__)
 
-CORS_OPTS = [
-    cfg.ListOpt('allowed_origin',
-                help='Indicate whether this resource may be shared with the '
-                     'domain received in the requests "origin" header. '
-                     'Format: "<protocol>://<host>[:<port>]", no trailing '
-                     'slash. Example: https://horizon.example.com'),
-    cfg.BoolOpt('allow_credentials',
-                default=True,
-                help='Indicate that the actual request can include user '
-                     'credentials'),
-    cfg.ListOpt('expose_headers',
-                default=[],
-                help='Indicate which headers are safe to expose to the API. '
-                     'Defaults to HTTP Simple Headers.'),
-    cfg.IntOpt('max_age',
-               default=3600,
-               help='Maximum cache age of CORS preflight requests.'),
-    cfg.ListOpt('allow_methods',
-                default=['OPTIONS', 'GET', 'HEAD', 'POST', 'PUT', 'DELETE',
-                         'TRACE', 'PATCH'],  # RFC 2616, RFC 5789
-                help='Indicate which methods can be used during the actual '
-                     'request.'),
-    cfg.ListOpt('allow_headers',
-                default=[],
-                help='Indicate which header field names may be used during '
-                     'the actual request.')
+OPTS = [
+    cfg.ListOpt(
+        'allowed_origin',
+        help='Indicate whether this resource may be shared with the '
+        'domain received in the requests "origin" header. '
+        'Format: "<protocol>://<host>[:<port>]", no trailing '
+        'slash. Example: https://horizon.example.com',
+    ),
+    cfg.BoolOpt(
+        'allow_credentials',
+        default=True,
+        help='Indicate that the actual request can include user credentials',
+    ),
+    cfg.ListOpt(
+        'expose_headers',
+        default=[],
+        help='Indicate which headers are safe to expose to the API. '
+        'Defaults to HTTP Simple Headers.',
+    ),
+    cfg.IntOpt(
+        'max_age',
+        default=3600,
+        help='Maximum cache age of CORS preflight requests.',
+    ),
+    cfg.ListOpt(
+        'allow_methods',
+        default=[
+            'OPTIONS',
+            'GET',
+            'HEAD',
+            'POST',
+            'PUT',
+            'DELETE',
+            'TRACE',
+            'PATCH',
+        ],  # RFC 2616, RFC 5789
+        help='Indicate which methods can be used during the actual request.',
+    ),
+    cfg.ListOpt(
+        'allow_headers',
+        default=[],
+        help='Indicate which header field names may be used during '
+        'the actual request.',
+    ),
 ]
 
+# legacy alias, since many projects reference this directly
+CORS_OPTS = OPTS
 
-def set_defaults(**kwargs):
+
+def set_defaults(**kwargs: cfg.Opt) -> None:
     """Override the default values for configuration options.
 
     This method permits a project to override the default CORS option values.
@@ -74,26 +110,26 @@ def set_defaults(**kwargs):
     # there's no good way for a user to override only one option, because all
     # the others would be overridden to 'None'.
 
-    valid_params = {k.name for k in CORS_OPTS
-                    if k.name != 'allowed_origin'}
+    valid_params = {k.name for k in OPTS if k.name != 'allowed_origin'}
     passed_params = {k for k in kwargs}
 
     wrong_params = passed_params - valid_params
     if wrong_params:
-        raise AttributeError('Parameter(s) [%s] invalid, please only use [%s]'
-                             % (wrong_params, valid_params))
+        raise AttributeError(
+            f'Parameter(s) [{wrong_params}] invalid, please only use '
+            f'[{valid_params}]'
+        )
 
     # Set global defaults.
-    cfg.set_defaults(CORS_OPTS, **kwargs)
+    cfg.set_defaults(OPTS, **kwargs)
 
 
 class InvalidOriginError(Exception):
     """Exception raised when Origin is invalid."""
 
-    def __init__(self, origin):
+    def __init__(self, origin: str):
         self.origin = origin
-        super().__init__(
-            'CORS request from origin \'%s\' not permitted.' % origin)
+        super().__init__(f"CORS request from origin '{origin}' not permitted.")
 
 
 class CORS(base.ConfigurableMiddleware):
@@ -113,23 +149,25 @@ class CORS(base.ConfigurableMiddleware):
         'Content-Language',
         'Expires',
         'Last-Modified',
-        'Pragma'
+        'Pragma',
     ]
 
-    def __init__(self, application, *args, **kwargs):
-        super().__init__(application, *args, **kwargs)
+    def __init__(
+        self,
+        application: WSGIApplication | None,
+        conf: dict[str, ty.Any] | cfg.ConfigOpts | None = None,
+    ) -> None:
+        super().__init__(application, conf)
         # Begin constructing our configuration hash.
-        self.allowed_origins = {}
+        self.allowed_origins: dict[str, AllowedOrigin] = {}
         self._init_conf()
 
-        def sanitize(csv_list):
-            try:
-                return [str.strip(x) for x in csv_list.split(',')]
-            except Exception:
-                return None
-
     @classmethod
-    def factory(cls, global_conf, **local_conf):
+    def factory(
+        cls: type[base.MiddlewareType],
+        global_conf: dict[str, ty.Any] | None,
+        **local_conf: ty.Any,
+    ) -> ty.Callable[[WSGIApplication], base.MiddlewareType]:
         """factory method for paste.deploy
 
         allowed_origin: Protocol, host, and port for the allowed origin.
@@ -139,17 +177,20 @@ class CORS(base.ConfigurableMiddleware):
         allow_methods: List of HTTP methods to permit.
         allow_headers: List of HTTP headers to permit from the client.
         """
-        if ('allowed_origin' not in local_conf and
-           'oslo_config_project' not in local_conf):
-            raise TypeError("allowed_origin or oslo_config_project "
-                            "is required")
+        if (
+            'allowed_origin' not in local_conf
+            and 'oslo_config_project' not in local_conf
+        ):
+            raise TypeError(
+                "allowed_origin or oslo_config_project is required"
+            )
         return super().factory(global_conf, **local_conf)
 
-    def _init_conf(self):
-        '''Initialize this middleware from an oslo.config instance.'''
+    def _init_conf(self) -> None:
+        """Initialize this middleware from an oslo.config instance."""
 
         # First, check the configuration and register global options.
-        self.oslo_conf.register_opts(CORS_OPTS, 'cors')
+        self.oslo_conf.register_opts(OPTS, 'cors')
 
         allowed_origin = self._conf_get('allowed_origin', 'cors')
         allow_credentials = self._conf_get('allow_credentials', 'cors')
@@ -158,44 +199,41 @@ class CORS(base.ConfigurableMiddleware):
         allow_methods = self._conf_get('allow_methods', 'cors')
         allow_headers = self._conf_get('allow_headers', 'cors')
 
-        # Clone our original CORS_OPTS, and set the defaults to whatever is
+        # Clone our original OPTS, and set the defaults to whatever is
         # set in the global conf instance. This is done explicitly (instead
         # of **kwargs), since we don't accidentally want to catch
         # allowed_origin.
-        subgroup_opts = copy.deepcopy(CORS_OPTS)
-        cfg.set_defaults(subgroup_opts,
-                         allow_credentials=allow_credentials,
-                         expose_headers=expose_headers,
-                         max_age=max_age,
-                         allow_methods=allow_methods,
-                         allow_headers=allow_headers)
+        subgroup_opts = copy.deepcopy(OPTS)
+        cfg.set_defaults(
+            subgroup_opts,
+            allow_credentials=allow_credentials,
+            expose_headers=expose_headers,
+            max_age=max_age,
+            allow_methods=allow_methods,
+            allow_headers=allow_headers,
+        )
 
         # If the default configuration contains an allowed_origin, don't
         # forget to register that.
-        self.add_origin(allowed_origin=allowed_origin,
-                        allow_credentials=allow_credentials,
-                        expose_headers=expose_headers,
-                        max_age=max_age,
-                        allow_methods=allow_methods,
-                        allow_headers=allow_headers)
+        self.add_origin(
+            allowed_origin=allowed_origin,
+            allow_credentials=allow_credentials,
+            expose_headers=expose_headers,
+            max_age=max_age,
+            allow_methods=allow_methods,
+            allow_headers=allow_headers,
+        )
 
-        # Iterate through all the loaded config sections, looking for ones
-        # prefixed with 'cors.'
-        for section in self.oslo_conf.list_all_sections():
-            if section.startswith('cors.'):
-                debtcollector.deprecate('Multiple configuration blocks are '
-                                        'deprecated and will be removed in '
-                                        'future versions. Please consolidate '
-                                        'your configuration in the [cors] '
-                                        'configuration block.')
-                # Register with the preconstructed defaults
-                self.oslo_conf.register_opts(subgroup_opts, section)
-                self.add_origin(**self.oslo_conf[section])
-
-    def add_origin(self, allowed_origin, allow_credentials=True,
-                   expose_headers=None, max_age=None, allow_methods=None,
-                   allow_headers=None):
-        '''Add another origin to this filter.
+    def add_origin(
+        self,
+        allowed_origin: list[str],
+        allow_credentials: bool = True,
+        expose_headers: list[str] | None = None,
+        max_age: int | None = None,
+        allow_methods: list[str] | None = None,
+        allow_headers: list[str] | None = None,
+    ) -> None:
+        """Add another origin to this filter.
 
         :param allowed_origin: Protocol, host, and port for the allowed origin.
         :param allow_credentials: Whether to permit credentials.
@@ -204,34 +242,31 @@ class CORS(base.ConfigurableMiddleware):
         :param allow_methods: List of HTTP methods to permit.
         :param allow_headers: List of HTTP headers to permit from the client.
         :return:
-        '''
-
-        # NOTE(dims): Support older code that still passes in
-        # a string for allowed_origin instead of a list
-        if isinstance(allowed_origin, str):
-            # TODO(krotscheck): https://review.opendev.org/#/c/312687/
-            LOG.warning('DEPRECATED: The `allowed_origin` keyword argument in '
-                        '`add_origin()` should be a list, found String.')
-            allowed_origin = [allowed_origin]
+        """
 
         if allowed_origin:
             for origin in allowed_origin:
-
                 if origin in self.allowed_origins:
-                    LOG.warning('Allowed origin [%s] already exists, skipping'
-                                % (allowed_origin,))
+                    LOG.warning(
+                        'Allowed origin [%s] already exists, skipping',
+                        allowed_origin,
+                    )
                     continue
 
                 self.allowed_origins[origin] = {
                     'allow_credentials': allow_credentials,
-                    'expose_headers': expose_headers,
+                    'expose_headers': expose_headers or [],
                     'max_age': max_age,
-                    'allow_methods': allow_methods,
-                    'allow_headers': allow_headers
+                    'allow_methods': allow_methods or [],
+                    'allow_headers': allow_headers or [],
                 }
 
-    def process_response(self, response, request=None):
-        '''Check for CORS headers, and decorate if necessary.
+    def process_response(
+        self,
+        response: webob.response.Response,
+        request: webob.request.Request | None = None,
+    ) -> webob.response.Response:
+        """Check for CORS headers, and decorate if necessary.
 
         Perform two checks. First, if an OPTIONS request was issued, let the
         application handle it, and (if necessary) decorate the response with
@@ -240,17 +275,22 @@ class CORS(base.ConfigurableMiddleware):
         OPTIONS requests, the response code is overridden.
 
         In the case of all other requests, regular request headers are applied.
-        '''
+        """
 
         # Sanity precheck: If we detect CORS headers provided by something in
         # in the middleware chain, assume that it knows better.
         if 'Access-Control-Allow-Origin' in response.headers:
             return response
 
+        # We need the request object
+        if not request:
+            return response
+
         # Doublecheck for an OPTIONS request.
         if request.method == 'OPTIONS':
-            return self._apply_cors_preflight_headers(request=request,
-                                                      response=response)
+            return self._apply_cors_preflight_headers(
+                request=request, response=response
+            )
 
         # Apply regular CORS headers.
         self._apply_cors_request_headers(request=request, response=response)
@@ -259,7 +299,10 @@ class CORS(base.ConfigurableMiddleware):
         return response
 
     @staticmethod
-    def _split_header_values(request, header_name):
+    def _split_header_values(
+        request: webob.request.Request,
+        header_name: str,
+    ) -> list[str]:
         """Convert a comma-separated header value into a list of values."""
         values = []
         if header_name in request.headers:
@@ -269,7 +312,11 @@ class CORS(base.ConfigurableMiddleware):
                     values.append(value)
         return values
 
-    def _apply_cors_preflight_headers(self, request, response):
+    def _apply_cors_preflight_headers(
+        self,
+        request: webob.request.Request,
+        response: webob.response.Response,
+    ) -> webob.response.Response:
         """Handle CORS Preflight (Section 6.2)
 
         Given a request and a response, apply the CORS preflight headers
@@ -280,7 +327,9 @@ class CORS(base.ConfigurableMiddleware):
         # underlying middleware's response content needs to be persisted.
         # Otherwise, create a new response.
         if 200 > response.status_code or response.status_code >= 300:
-            response = base.NoContentTypeResponse(status=webob.exc.HTTPOk.code)
+            response = base.NoContentTypeResponse(
+                status=str(webob.exc.HTTPOk.code)
+            )
 
         # Does the request have an origin header? (Section 6.2.1)
         if 'Origin' not in request.headers:
@@ -289,22 +338,25 @@ class CORS(base.ConfigurableMiddleware):
         # Is this origin registered? (Section 6.2.2)
         try:
             origin, cors_config = self._get_cors_config_by_origin(
-                request.headers['Origin'])
+                request.headers['Origin']
+            )
         except InvalidOriginError:
             return response
 
         # If there's no request method, exit. (Section 6.2.3)
         if 'Access-Control-Request-Method' not in request.headers:
-            LOG.debug('CORS request does not contain '
-                      'Access-Control-Request-Method header.')
+            LOG.debug(
+                'CORS request does not contain '
+                'Access-Control-Request-Method header.'
+            )
             return response
         request_method = request.headers['Access-Control-Request-Method']
 
         # Extract Request headers. If parsing fails, exit. (Section 6.2.4)
         try:
-            request_headers = \
-                self._split_header_values(request,
-                                          'Access-Control-Request-Headers')
+            request_headers = self._split_header_values(
+                request, 'Access-Control-Request-Headers'
+            )
         except Exception:
             LOG.debug('Cannot parse request headers.')
             return response
@@ -312,20 +364,27 @@ class CORS(base.ConfigurableMiddleware):
         # Compare request method to permitted methods (Section 6.2.5)
         permitted_methods = cors_config['allow_methods']
         if request_method not in permitted_methods:
-            LOG.debug('Request method \'%s\' not in permitted list: %s'
-                      % (request_method, permitted_methods))
+            LOG.debug(
+                'Request method \'%s\' not in permitted list: %s',
+                request_method,
+                permitted_methods,
+            )
             return response
 
         # Compare request headers to permitted headers, case-insensitively.
         # (Section 6.2.6)
-        permitted_headers = [header.upper() for header in
-                             (cors_config['allow_headers'] +
-                              self.simple_headers)]
+        permitted_headers = [
+            header.upper()
+            for header in (cors_config['allow_headers'] + self.simple_headers)
+        ]
         for requested_header in request_headers:
             upper_header = requested_header.upper()
             if upper_header not in permitted_headers:
-                LOG.debug('Request header \'%s\' not in permitted list: %s'
-                          % (requested_header, permitted_headers))
+                LOG.debug(
+                    'Request header \'%s\' not in permitted list: %s',
+                    requested_header,
+                    permitted_headers,
+                )
                 return response
 
         # Set the default origin permission headers. (Sections 6.2.7, 6.4)
@@ -338,30 +397,40 @@ class CORS(base.ConfigurableMiddleware):
 
         # Attach Access-Control-Max-Age if appropriate. (Section 6.2.8)
         if 'max_age' in cors_config and cors_config['max_age']:
-            response.headers['Access-Control-Max-Age'] = \
-                str(cors_config['max_age'])
+            response.headers['Access-Control-Max-Age'] = str(
+                cors_config['max_age']
+            )
 
         # Attach Access-Control-Allow-Methods. (Section 6.2.9)
         response.headers['Access-Control-Allow-Methods'] = request_method
 
         # Attach  Access-Control-Allow-Headers. (Section 6.2.10)
         if request_headers:
-            response.headers['Access-Control-Allow-Headers'] = \
-                ','.join(request_headers)
+            response.headers['Access-Control-Allow-Headers'] = ','.join(
+                request_headers
+            )
 
         return response
 
-    def _get_cors_config_by_origin(self, origin):
+    def _get_cors_config_by_origin(
+        self,
+        origin: str,
+    ) -> tuple[str, AllowedOrigin]:
         if origin not in self.allowed_origins:
             if '*' in self.allowed_origins:
                 origin = '*'
             else:
-                LOG.debug('CORS request from origin \'%s\' not permitted.'
-                          % origin)
+                LOG.debug(
+                    'CORS request from origin \'%s\' not permitted.', origin
+                )
                 raise InvalidOriginError(origin)
         return origin, self.allowed_origins[origin]
 
-    def _apply_cors_request_headers(self, request, response):
+    def _apply_cors_request_headers(
+        self,
+        request: webob.request.Request,
+        response: webob.response.Response,
+    ) -> None:
         """Handle Basic CORS Request (Section 6.1)
 
         Given a request and a response, apply the CORS headers appropriate
@@ -375,7 +444,8 @@ class CORS(base.ConfigurableMiddleware):
         # Is this origin registered? (Section 6.1.2)
         try:
             origin, cors_config = self._get_cors_config_by_origin(
-                request.headers['Origin'])
+                request.headers['Origin']
+            )
         except InvalidOriginError:
             return
 
@@ -392,8 +462,9 @@ class CORS(base.ConfigurableMiddleware):
 
         # Attach the exposed headers and exit. (Section 6.1.4)
         if cors_config['expose_headers']:
-            response.headers['Access-Control-Expose-Headers'] = \
-                ','.join(cors_config['expose_headers'])
+            response.headers['Access-Control-Expose-Headers'] = ','.join(
+                cors_config['expose_headers']
+            )
 
 
 # NOTE(sileht): Shortcut for backwards compatibility
