@@ -26,7 +26,6 @@ import sys
 import traceback
 import typing as ty
 
-from debtcollector import removals
 import jinja2
 from oslo_utils import reflection
 from oslo_utils import timeutils
@@ -39,7 +38,7 @@ import webob.response
 try:
     import greenlet
 except ImportError:
-    greenlet = None
+    greenlet = None  # type: ignore
 
 from oslo_middleware import base
 from oslo_middleware.exceptions import ConfigInvalid
@@ -265,7 +264,7 @@ class Healthcheck(base.ConfigurableMiddleware):
         [pipeline:public_api]
         pipeline = healthcheck sizelimit [...] public_service
 
-    Multiple filter sections can be defined if it desired to have
+    Multiple app sections can be defined if it desired to have
     pipelines with different healthcheck configuration, example:
 
     .. code-block:: ini
@@ -291,11 +290,11 @@ class Healthcheck(base.ConfigurableMiddleware):
         backends = disable_by_file
         disable_by_file_path = /var/run/nova/healthcheck_public_disable
 
-        [filter:healthcheck_admin]
+        [app:healthcheck_admin]
         use = egg:oslo.middleware:healthcheck
         backends = disable_by_file
         disable_by_file_path = /var/run/nova/healthcheck_admin_disable
-    """
+    """  # noqa: E501
 
     NAMESPACE = "oslo.middleware.healthcheck"
     HEALTHY_TO_STATUS_CODES = {
@@ -406,7 +405,6 @@ Reason
         self.oslo_conf.register_opts(
             opts.HEALTHCHECK_OPTS, group='healthcheck'
         )
-        self._path = self._conf_get('path')
         self._show_details = self._conf_get('detailed')
         self._source_ranges = [
             ipaddress.ip_network(r)
@@ -420,6 +418,10 @@ Reason
         # `disable_by_file` backends are not enabled at same time.
         self._verify_configured_plugins()
 
+        if ty.TYPE_CHECKING:
+            self._backends: stevedore.NamedExtensionManager[
+                pluginbase.HealthcheckBaseExtension
+            ]
         self._backends = stevedore.NamedExtensionManager(
             self.NAMESPACE,
             self._conf_get('backends'),
@@ -440,7 +442,6 @@ Reason
         # always return text/plain (because sending an error from this
         # middleware actually can cause issues).
         self._default_accept = 'text/plain'
-        self._ignore_path = False
 
     def _verify_configured_plugins(self) -> None:
         backends = self._conf_get('backends')
@@ -456,17 +457,15 @@ Reason
     def _conf_get(self, key: str, group: str = 'healthcheck') -> ty.Any:
         return super()._conf_get(key, group=group)
 
-    @removals.remove(  # type: ignore
-        message="The healthcheck middleware must now be configured as "
-        "an application, not as a filter"
-    )
     @classmethod
     def factory(
-        cls,
+        cls: type[base.MiddlewareType],
         global_conf: dict[str, ty.Any] | None,
         **local_conf: ty.Any,
-    ) -> ty.Callable[[WSGIApplication], base.ConfigurableMiddleware]:
-        return super().factory(global_conf, **local_conf)
+    ) -> ty.Callable[[WSGIApplication], base.MiddlewareType]:
+        raise NotImplementedError(
+            'HealthcheckMiddleware should be deployed as an app, not a filter'
+        )
 
     @classmethod
     def app_factory(
@@ -486,7 +485,6 @@ Reason
         conf = global_conf.copy() if global_conf else {}
         conf.update(local_conf)
         middleware = cls(None, conf)
-        middleware._ignore_path = True
         return middleware
 
     @staticmethod
@@ -629,9 +627,6 @@ Reason
         self,
         req: webob.request.Request,
     ) -> webob.response.Response | None:
-        if not self._ignore_path and req.path != self._path:
-            return None
-
         if self._source_ranges:
             if not req.remote_addr:
                 return None
@@ -657,7 +652,9 @@ Reason
                     return None
 
         results = [
-            ext.obj.healthcheck(req.server_port) for ext in self._backends
+            ext.obj.healthcheck(req.server_port)
+            for ext in self._backends
+            if ext.obj
         ]
         healthy = self._are_results_healthy(results)
         if req.method == "HEAD":
